==> sdb-2.0.1.tar.gz: sdb/src/Plugin/Block/SimpleDateBlock.php <==
<?php

namespace Drupal\simple_date_block\Plugin\Block;

use Drupal\Component\Datetime\DateTimePlus;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Provides a configurable date and time block.
 *
 * @Block(
 *   id = "simple_date_block",
 *   admin_label = @Translation("Simple Date Block"),
 *   category = @Translation("Custom")
 * )
 */
class SimpleDateBlock extends BlockBase implements BlockPluginInterface {

  /**
   * Default date format.
   */
  const DEFAULT_DATE_FORMAT = 'd/m/Y';

  /**
   * Common date format options.
   */
  const DATE_FORMATS = [
    'd/m/Y' => 'Day/Month/Year (e.g., 23/04/2025)',
    'Y-m-d' => 'Year-Month-Day (e.g., 2025-04-23)',
    'F j, Y' => 'Month Day, Year (e.g., April 23, 2025)',
    'd M Y' => 'Short Month (e.g., 23 Apr 2025)',
    'd/m/Y H:i:s' => 'Date and Time (e.g., 23/04/2025 14:30:00)',
    'custom' => 'Custom format',
  ];

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $format = $config['simple_date_block_format'] ?? self::DEFAULT_DATE_FORMAT;
    $timezone = $config['simple_date_block_timezone'] ?? \Drupal::config('system.date')->get('timezone')['default'];
    $relative = $config['simple_date_block_relative'] ?? FALSE;
    $timestamp = !empty($config['simple_date_block_custom_date']) && is_numeric($config['simple_date_block_custom_date'])
      ? (int) $config['simple_date_block_custom_date']
      : time();

    $cache = [
      'max-age' => $relative ? 10 : 60, // Shorter cache for relative time.
      'contexts' => ['timezone', 'languages:language_interface'],
      'tags' => ['simple_date_block'],
    ];

    if ($relative) {
      $output = \Drupal::service('date.formatter')->formatInterval(time() - $timestamp, 2);
    }
    else {
      try {
        $output = \Drupal::service('date.formatter')->format($timestamp, 'custom', $format, $timezone);
      }
      catch (\Exception $e) {
        \Drupal::logger('simple_date_block')->error('Invalid date format: @format', ['@format' => $format]);
        $output = \Drupal::service('date.formatter')->format($timestamp, 'custom', self::DEFAULT_DATE_FORMAT, $timezone);
      }
    }

    return [
      '#theme' => 'simple_date_block',
      '#date' => $output,
      '#cache' => $cache,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    // Relative time option.
    $form['simple_date_block_relative'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display relative time'),
      '#description' => $this->t('Show the time as a relative string (e.g., "5 minutes ago") instead of a formatted date.'),
      '#default_value' => $config['simple_date_block_relative'] ?? FALSE,
    ];

    // Date format selection.
    $form['simple_date_block_format_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#options' => array_combine(array_keys(self::DATE_FORMATS), array_column(self::DATE_FORMATS, 1)),
      '#default_value' => $config['simple_date_block_format_type'] ?? 'd/m/Y',
      '#ajax' => [
        'callback' => [$this, 'updateFormatPreview'],
        'wrapper' => 'date-format-preview',
      ],
      '#states' => [
        'disabled' => [
          ':input[name="settings[simple_date_block_relative]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Custom format field (visible if 'custom' is selected and relative time is off).
    $form['simple_date_block_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date format'),
      '#description' => $this->t('Enter a custom PHP date format. See <a href=":link">PHP date format documentation</a> for details. Examples: "d/m/Y" (23/04/2025), "Y-m-d" (2025-04-23), "d/m/Y H:i:s" (23/04/2025 14:30:00).', [
        ':link' => 'https://www.php.net/manual/en/datetime.format.php',
      ]),
      '#default_value' => $config['simple_date_block_format'] ?? self::DEFAULT_DATE_FORMAT,
      '#states' => [
        'visible' => [
          ':input[name="settings[simple_date_block_format_type]"]' => ['value' => 'custom'],
          ':input[name="settings[simple_date_block_relative]"]' => ['checked' => FALSE],
        ],
      ],
      '#maxlength' => 20,
    ];

    // Timezone selection.
    $form['simple_date_block_timezone'] = [
      '#type' => 'select',
      '#title' => $this->t('Timezone'),
      '#options' => system_time_zones(),
      '#default_value' => $config['simple_date_block_timezone'] ?? \Drupal::config('system.date')->get('timezone')['default'],
      '#description' => $this->t('Select the timezone for the date display.'),
    ];

    // Custom date source.
    $form['simple_date_block_custom_date'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date (timestamp)'),
      '#description' => $this->t('Enter a Unix timestamp for a specific date (e.g., 1697059200 for Oct 12, 2023). Leave blank to use the current time.'),
      '#default_value' => $config['simple_date_block_custom_date'] ?? '',
    ];

    // Format preview.
    $form['preview'] = [
      '#type' => 'container',
      '#id' => 'date-format-preview',
      '#markup' => $this->getFormatPreview($form_state, $config),
    ];

    return $form;
  }

  /**
   * AJAX callback to update the date format preview.
   */
  public function updateFormatPreview(array &$form, FormStateInterface $form_state) {
    return $form['preview'];
  }

  /**
   * Generates a preview of the selected date format.
   */
  protected function getFormatPreview(FormStateInterface $form_state, array $config) {
    $relative = $form_state->getValue('simple_date_block_relative', $config['simple_date_block_relative'] ?? FALSE);
    $format_type = $form_state->getValue('simple_date_block_format_type', $config['simple_date_block_format_type'] ?? 'd/m/Y');
    $format = $format_type === 'custom' ? $form_state->getValue('simple_date_block_format', $config['simple_date_block_format'] ?? self::DEFAULT_DATE_FORMAT) : $format_type;
    $timestamp = !empty($form_state->getValue('simple_date_block_custom_date', $config['simple_date_block_custom_date']))
      && is_numeric($form_state->getValue('simple_date_block_custom_date', $config['simple_date_block_custom_date']))
      ? (int) $form_state->getValue('simple_date_block_custom_date', $config['simple_date_block_custom_date'])
      : time();

    if ($relative) {
      return $this->t('Preview: @date', ['@date' => \Drupal::service('date.formatter')->formatInterval(time() - $timestamp, 2)]);
    }

    try {
      $date = \Drupal::service('date.formatter')->format($timestamp, 'custom', $format);
      return $this->t('Preview: @date', ['@date' => $date]);
    }
    catch (\Exception $e) {
      return $this->t('Invalid format. Please check the date format.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    // Validate custom date format.
    $format_type = $form_state->getValue('simple_date_block_format_type');
    $relative = $form_state->getValue('simple_date_block_relative');
    if ($format_type === 'custom' && !$relative) {
      $format = $form_state->getValue('simple_date_block_format');
      try {
        \Drupal::service('date.formatter')->format(time(), 'custom', $format);
      }
      catch (\Exception $e) {
        $form_state->setErrorByName('simple_date_block_format', $this->t('The custom date format is invalid. Please provide a valid format.'));
      }
    }

    // Validate custom timestamp.
    $custom_date = $form_state->getValue('simple_date_block_custom_date');
    if (!empty($custom_date) && (!is_numeric($custom_date) || $custom_date < 0)) {
      $form_state->setErrorByName('simple_date_block_custom_date', $this->t('The custom date must be a valid Unix timestamp (a positive integer).'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('simple_date_block_relative', $form_state->getValue('simple_date_block_relative'));
    $this->setConfigurationValue('simple_date_block_format_type', $form_state->getValue('simple_date_block_format_type'));
    $this->setConfigurationValue('simple_date_block_format', $form_state->getValue('simple_date_block_format_type') === 'custom' ? $form_state->getValue('simple_date_block_format') : $form_state->getValue('simple_date_block_format_type'));
    $this->setConfigurationValue('simple_date_block_timezone', $form_state->getValue('simple_date_block_timezone'));
    $this->setConfigurationValue('simple_date_block_custom_date', $form_state->getValue('simple_date_block_custom_date'));
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return Cache::mergeTags(parent::getCacheTags(), ['simple_date_block']);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return Cache::mergeContexts(parent::getCacheContexts(), ['timezone', 'languages:language_interface']);
  }
}

==> sdb-2.0.2.tar.gz: sdb/src/Plugin/Block/SimpleDateBlock.php <==
<?php

namespace Drupal\simple_date_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a configurable date and time block.
 *
 * @Block(
 *   id = "simple_date_block",
 *   admin_label = @Translation("Simple Date Block"),
 *   category = @Translation("Custom")
 * )
 */
class SimpleDateBlock extends BlockBase implements BlockPluginInterface {

  /**
   * Default date format.
   */
  const DEFAULT_DATE_FORMAT = 'd/m/Y';

  /**
   * Common date format options.
   */
  const DATE_FORMATS = [
    'd/m/Y' => 'Day/Month/Year (e.g., 23/04/2025)',
    'Y-m-d' => 'Year-Month-Day (e.g., 2025-04-23)',
    'F j, Y' => 'Month Day, Year (e.g., April 23, 2025)',
    'd M Y' => 'Short Month (e.g., 23 Apr 2025)',
    'd/m/Y H:i:s' => 'Date and Time (e.g., 23/04/2025 14:30:00)',
    'custom' => 'Custom format',
  ];

  /**
   * Builds the block’s render array.
   *
   * @return array
   *   Render array with formatted date.
   */
  public function build() {
    $config = $this->getConfiguration();
    $format = $config['simple_date_block_format'] ?? self::DEFAULT_DATE_FORMAT;
    $timezone = $config['simple_date_block_timezone'] ?? \Drupal::config('system.date')->get('timezone')['default'];
    $relative = $config['simple_date_block_relative'] ?? FALSE;
    $timestamp = !empty($config['simple_date_block_custom_date']) && is_numeric($config['simple_date_block_custom_date'])
      ? (int) $config['simple_date_block_custom_date']
      : time();

    $cache = [
      'max-age' => $relative ? 10 : 60,
      'contexts' => ['timezone', 'languages:language_interface'],
      'tags' => ['simple_date_block'],
    ];

    if ($relative) {
      $output = \Drupal::service('date.formatter')->formatInterval(time() - $timestamp, 2);
    }
    else {
      try {
        $output = \Drupal::service('date.formatter')->format($timestamp, 'custom', $format, $timezone);
      }
      catch (\Exception $e) {
        \Drupal::logger('simple_date_block')->error('Invalid date format: @format', ['@format' => $format]);
        $output = \Drupal::service('date.formatter')->format($timestamp, 'custom', self::DEFAULT_DATE_FORMAT, $timezone);
      }
    }

    return [
      '#theme' => 'simple_date_block',
      '#date' => $output,
      '#cache' => $cache,
    ];
  }

  /**
   * Builds the block configuration form.
   *
   * @param array $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return array
   *   Modified form array.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['simple_date_block_relative'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display relative time'),
      '#description' => $this->t('Show the time as a relative string (e.g., "5 minutes ago") instead of a formatted date.'),
      '#default_value' => $config['simple_date_block_relative'] ?? FALSE,
    ];

    $form['simple_date_block_format_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#options' => array_combine(array_keys(self::DATE_FORMATS), array_values(self::DATE_FORMATS)),
      '#default_value' => $config['simple_date_block_format_type'] ?? 'd/m/Y',
      '#states' => [
        'disabled' => [
          ':input[name="settings[simple_date_block_relative]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['simple_date_block_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date format'),
      '#description' => $this->t('Enter a custom PHP date format. See <a href=":link">PHP date format documentation</a> for details. Examples: "d/m/Y" (23/04/2025), "Y-m-d" (2025-04-23).', [
        ':link' => 'https://www.php.net/manual/en/datetime.format.php',
      ]),
      '#default_value' => $config['simple_date_block_format'] ?? self::DEFAULT_DATE_FORMAT,
      '#states' => [
        'visible' => [
          ':input[name="settings[simple_date_block_format_type]"]' => ['value' => 'custom'],
          ':input[name="settings[simple_date_block_relative]"]' => ['checked' => FALSE],
        ],
      ],
      '#maxlength' => 20,
    ];

    $timezones = function_exists('system_time_zones') ? system_time_zones() : $this->getFallbackTimezones();
    $form['simple_date_block_timezone'] = [
      '#type' => 'select',
      '#title' => $this->t('Timezone'),
      '#options' => $timezones,
      '#default_value' => $config['simple_date_block_timezone'] ?? \Drupal::config('system.date')->get('timezone')['default'],
      '#description' => $this->t('Select the timezone for the date display.'),
    ];

    $form['simple_date_block_custom_date'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date (timestamp)'),
      '#description' => $this->t('Enter a Unix timestamp for a specific date (e.g., 1697059200 for Oct 12, 2023). Leave blank to use the current time.'),
      '#default_value' => $config['simple_date_block_custom_date'] ?? '',
    ];

    return $form;
  }

  /**
   * Provides a fallback list of timezones if system_time_zones() is unavailable.
   *
   * @return array
   *   Timezone identifiers.
   */
  protected function getFallbackTimezones() {
    $timezones = \DateTimeZone::listIdentifiers(\DateTimeZone::ALL);
    return array_combine($timezones, $timezones);
  }

  /**
   * Validates the block configuration form.
   *
   * @param array $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    $format_type = $form_state->getValue('simple_date_block_format_type');
    $relative = $form_state->getValue('simple_date_block_relative');
    if ($format_type === 'custom' && !$relative) {
      $format = $form_state->getValue('simple_date_block_format');
      try {
        \Drupal::service('date.formatter')->format(time(), 'custom', $format);
      }
      catch (\Exception $e) {
        $form_state->setErrorByName('simple_date_block_format', $this->t('The custom date format is invalid. Please provide a valid format.'));
      }
    }

    $custom_date = $form_state->getValue('simple_date_block_custom_date');
    if (!empty($custom_date) && (!is_numeric($custom_date) || $custom_date < 0)) {
      $form_state->setErrorByName('simple_date_block_custom_date', $this->t('The custom date must be a valid Unix timestamp (a positive integer).'));
    }
  }

  /**
   * Saves the block configuration.
   *
   * @param array $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('simple_date_block_relative', $form_state->getValue('simple_date_block_relative'));
    $this->setConfigurationValue('simple_date_block_format_type', $form_state->getValue('simple_date_block_format_type'));
    $this->setConfigurationValue('simple_date_block_format', $form_state->getValue('simple_date_block_format_type') === 'custom' ? $form_state->getValue('simple_date_block_format') : $form_state->getValue('simple_date_block_format_type'));
    $this->setConfigurationValue('simple_date_block_timezone', $form_state->getValue('simple_date_block_timezone'));
    $this->setConfigurationValue('simple_date_block_custom_date', $form_state->getValue('simple_date_block_custom_date'));
  }

  /**
   * Provides cache tags for the block.
   *
   * @return array
   *   Cache tags.
   */
  public function getCacheTags() {
    return Cache::mergeTags(parent::getCacheTags(), ['simple_date_block']);
  }

  /**
   * Provides cache contexts for the block.
   *
   * @return array
   *   Cache contexts.
   */
  public function getCacheContexts() {
    return Cache::mergeContexts(parent::getCacheContexts(), ['timezone', 'languages:language_interface']);
  }
}
